local addonName, APP = ...

local frame = CreateFrame("Frame")
frame:RegisterEvent("ARTIFACT_UPDATE")
frame:RegisterEvent("ARTIFACT_CLOSE")
-- APP.frame = frame

local currentArtifact = nil
local powerButtonOverlays = {}
--[[
powerButtonOverlays structure
powerButtonOverlays = {
	[artifact] = {
		[powerID] = overlay,
	},
}
]]

local debugMode = false
local function Debug(msg)
	if debugMode then print(msg) end
end

-- function PathSelectionDropDown_Initialize(self, level)
	-- local info = UIDropDownMenu_CreateInfo()
	-- info.text = 
	-- info.value = 
	-- UIDropDownMenu_AddButton(info, level);
-- end

-- local pathSelectionMenu
-- local function SetPathSelectionDropDown()
	-- if not pathSelectionMenu then
		-- pathSelectionMenu = CreateFrame("Frame", "APP_PathSelectionMenu", ArtifactFrame, "UIDropDownMenuTemplate")
		-- pathSelectionMenu:SetPoint("TOPRIGHT", 0, -50)
	-- end
-- end

-- Create overlays for specific artifact
local function CreateButtonOverlays(artifact)
	Debug("CreateButtonOverlays:" .. (artifact or "nil") .. ", " .. ((not powerButtonOverlays[artifact]) and "create" or "exist"))
	if not powerButtonOverlays[artifact] then	-- not initialized
		powerButtonOverlays[artifact] = {}
		
		-- SocketInventoryItem(_G.INVSLOT_MAINHAND)
		-- local powerButtons = ArtifactFrame.PerksTab.PowerButtons

		local powers = C_ArtifactUI.GetPowers()
		if powers then	-- artifact ready
			Debug("#C_ArtifactUI.GetPowers():" .. #powers)
			for i = 1, #powers do
				local powerButton = ArtifactFrame.PerksTab:GetOrCreatePowerButton(i)
				local powerID = powers[i]
				local overlay = powerButton:CreateFontString(nil, "OVERLAY")
				local font = GameFontNormal:GetFont()
				overlay:SetFont(font, 17, "OUTLINE")
				overlay:SetPoint("CENTER", 0, 0)
				powerButtonOverlays[artifact][powerID] = overlay
			end
			wipe(powers)
		else	-- artifact not ready
			-- When will we meet this?
		end
	end
end

local function SetOverlaysText(artifact)
	Debug("SetOverlaysText:" .. (artifact or "nil"))
	for i = 1, #APP.data[artifact] do
		local powerID = APP.data[artifact][i]
		local overlay = powerButtonOverlays[artifact][powerID]
		if overlay then
			overlay:SetText(i)
			overlay:Show()
			Debug("SetText:" .. i .. " - " .. powerID)
		else
			print("InvalidPowerID:" .. powerID)
		end
	end
end

local function SetButtonHighlight(artifact)
	-- local temp = APP.data[artifact]
	-- if #temp = 0 then -- has 2 paths at least
		-- temp = APP.data[artifact]
	-- end
	for i = 1, #APP.data[artifact] do
		local powerID = APP.data[artifact][i]
		-- spellID, cost, currentRank, maxRank, bonusRanks, x, y, prereqsMet, isStart, isGoldMedal, isFinal = C_ArtifactUI.GetPowerInfo(powerID)
		local _, _, currentRank, maxRank = C_ArtifactUI.GetPowerInfo(powerID)
		local overlay = powerButtonOverlays[artifact][powerID]
		local button = overlay:GetParent()
		
		if currentRank < maxRank then
			Debug("SetHighlight:" .. powerID)
			-- BigGlow, BigWhirls, RingBurst, RingGlow, SpinningGlows, StarBurst, WhiteStarBurst ...
			button.BigGlow:SetAlpha(1)
			button.RingGlow:SetAlpha(1)
			overlay:SetTextColor(1, 0.3, 0.3, 1)
			return
		else	-- restore changes
			-- Debug("RemoveHightlight:" .. powerID)
			button.BigGlow:SetAlpha(0)
			button.RingGlow:SetAlpha(0)
			overlay:SetTextColor(1, 1, 1, 1)
		end
	end
end

local function APP_HideAll()
	Debug("APP_HideAll...")
	for artifactID, artifactTable in pairs(powerButtonOverlays) do
		Debug("APP_HideAll:" .. artifactID)
		for _, overlay in pairs(artifactTable) do
			overlay:Hide()
			overlay:GetParent().BigGlow:SetAlpha(0)
			overlay:GetParent().RingGlow:SetAlpha(0)
		end
	end
end

local function APP_HideCurrent()
	if powerButtonOverlays[currentArtifact] then
		Debug("APP_HideCurrent:" .. currentArtifact)
		for _, overlay in pairs(powerButtonOverlays[currentArtifact]) do
			overlay:Hide()
			overlay:GetParent().BigGlow:SetAlpha(0)
			overlay:GetParent().RingGlow:SetAlpha(0)
		end
	else
		APP_HideAll()
	end
end

local function APP_Refresh()
	-- itemID, altItemID, name, iconFileDataID, powerAvailable, ranksPurchased, ranksKnown, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop = C_ArtifactUI.GetArtifactInfo()
	currentArtifact = C_ArtifactUI.GetArtifactInfo()
	if currentArtifact then
		APP_HideAll()
		
		Debug("APP_Refresh:" .. currentArtifact .. (APP.data[currentArtifact] and ", supported" or ", not supported"))
		if APP.data[currentArtifact] then	-- supported
		
			CreateButtonOverlays(currentArtifact)
			SetOverlaysText(currentArtifact)
			SetButtonHighlight(currentArtifact)
		end
	end
end

function frame:ARTIFACT_UPDATE()
	APP_Refresh()
end

function frame:ARTIFACT_CLOSE()
	APP_HideCurrent()
end

frame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)